@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2005-2018 Intel Corporation
+    Copyright (c) 2005-2020 Intel Corporation
 
     Licensed under the Apache License, Version 2.0 (the "License");
     you may not use this file except in compliance with the License.
@@ -12,21 +12,17 @@
     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
     See the License for the specific language governing permissions and
     limitations under the License.
-
-
-
-
 */
 
 #ifndef __TBB_tbb_stddef_H
 #define __TBB_tbb_stddef_H
 
 // Marketing-driven product version
-#define TBB_VERSION_MAJOR 2018
-#define TBB_VERSION_MINOR 0
+#define TBB_VERSION_MAJOR 2020
+#define TBB_VERSION_MINOR 3
 
 // Engineering-focused interface version
-#define TBB_INTERFACE_VERSION 10005
+#define TBB_INTERFACE_VERSION 11103
 #define TBB_INTERFACE_VERSION_MAJOR TBB_INTERFACE_VERSION/1000
 
 // The oldest major interface version still supported
@@ -246,6 +242,14 @@ const size_t NFS_MaxLineSize = 128;
 #define __TBB_override // formal comment only
 #endif
 
+#if __TBB_CPP17_FALLTHROUGH_PRESENT
+#define __TBB_fallthrough [[fallthrough]]
+#elif __TBB_FALLTHROUGH_PRESENT
+#define __TBB_fallthrough __attribute__ ((fallthrough))
+#else
+#define __TBB_fallthrough
+#endif
+
 template<class T, size_t S, size_t R>
 struct padded_base : T {
     char pad[S - R];
@@ -312,6 +316,25 @@ inline T punned_cast( U* ptr ) {
     return reinterpret_cast<T>(x);
 }
 
+#if __TBB_DEFAULTED_AND_DELETED_FUNC_PRESENT
+
+//! Base class for types that should not be assigned.
+class no_assign {
+public:
+    void operator=( const no_assign& ) = delete;
+    no_assign( const no_assign& ) = default;
+    no_assign() = default;
+};
+
+//! Base class for types that should not be copied or assigned.
+class no_copy: no_assign {
+public:
+    no_copy( const no_copy& ) = delete;
+    no_copy() = default;
+};
+
+#else /*__TBB_DEFAULTED_AND_DELETED_FUNC_PRESENT*/
+
 //! Base class for types that should not be assigned.
 class no_assign {
     // Deny assignment
@@ -332,6 +355,8 @@ public:
     no_copy() {}
 };
 
+#endif /*__TBB_DEFAULTED_AND_DELETED_FUNC_PRESENT*/
+
 #if TBB_DEPRECATED_MUTEX_COPYING
 class mutex_copy_deprecated_and_disabled {};
 #else
@@ -423,8 +448,8 @@ private:
 // Following is a set of classes and functions typically used in compile-time "metaprogramming".
 // TODO: move all that to a separate header
 
-#if __TBB_ALLOCATOR_TRAITS_PRESENT || __TBB_CPP11_SMART_POINTERS_PRESENT
-#include <memory> // for allocator_traits, unique_ptr
+#if __TBB_CPP11_SMART_POINTERS_PRESENT
+#include <memory> // for unique_ptr
 #endif
 
 #if __TBB_CPP11_RVALUE_REF_PRESENT || __TBB_CPP11_DECLTYPE_PRESENT || _LIBCPP_VERSION
@@ -464,15 +489,6 @@ struct bool_constant {
 typedef bool_constant<true> true_type;
 typedef bool_constant<false> false_type;
 
-#if __TBB_ALLOCATOR_TRAITS_PRESENT
-using std::allocator_traits;
-#else
-template<typename allocator>
-struct allocator_traits{
-    typedef tbb::internal::false_type propagate_on_container_move_assignment;
-};
-#endif
-
 //! A template to select either 32-bit or 64-bit constant as compile time, depending on machine word size.
 template <unsigned u, unsigned long long ull >
 struct select_size_t_constant {
